#!/usr/bin/python3

import requests
from feii.log import Log
from feii.config import Config
from feii.request import Request

class Cluster(Config, Request):
  def __init__(self,
    cluster: str = {},
  ):
    super().__init__()
    self.cluster = cluster

  def get_status_cluster(self):
    self.cluster = requests.get("{0}/_cluster/health".format( self.ELASTIC_URL )).json()

  def check_status_cluster_not_red(self):
    return self.cluster['status'] != "red"

  def check_count_relocating_shards_in_cluster(self):
    return int(self.cluster['relocating_shards']) <= self.MAX_RELOCATING

  def check_count_pending_tasks_in_cluster(self):
    return int(self.cluster['number_of_pending_tasks']) <= self.MAX_TASKS

  def retry_failed(self):
    self.request = requests.post("{0}/_cluster/reroute?retry_failed".format( self.ELASTIC_URL ))

  def check_retry_failed(self):
    if self.status_request():
      self.logger.info("Retry_failed for index is over")
      return True



#!/usr/bin/python3

from feii.log import Log
from feii.config import Config
from feii.init import Init
from feii.structure import Structure

class Rollover(Structure):
  def rollover_last_index(self):
    for index in self.last_indices:
      self.index = index['index']
      self.alias = index['index.alias']
      self.logger.warning("Index [{0}][{1}gb] is larger the allowed size - {2}gb".format( self.index, int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))

      if not self.rollover_index_and_check():
        self.logger.error("Not rollover index for alias [{0}]".format( self.alias ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.not_next_step_for_index_and_check()

  def rollover_not_last_index(self):
    for index in self.not_last_indices:
      self.index = index['index']
      self.alias = index['index.alias']
      self.logger.warning("Index [{0}][{1}gb] is larger the allowed size - {2}gb".format( self.index, int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))

      self.index = self.find_next_index()
      if not self.add_write_disable_for_index_and_check():
        self.logger.error("Failed add alias [{0}] to next index [{1}] and disable write".format( self.alias, self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.index = index['index']
      if not self.add_write_disable_for_index_and_check():
        self.logger.error("Failed disable write to index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.index = self.find_next_index()
      if not self.add_write_enable_for_index_and_check():
        self.logger.error("Failed enable write to next index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      if not self.ilm_retry_index_and_check():
        self.logger.error("Failed ilm retry to next index [{0}]".format( self.index ))

      self.index = index['index']
      self.not_next_step_for_index_and_check()

  def rollover_last_shrink_index(self):
    for index in self.last_shrink_indices:
      self.index = index['index']
      self.alias = index['index.alias']
      self.logger.warning("Index [{0}][{1}gb] is larger the allowed size - {2}gb".format( self.index, int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))

      if not self.create_new_index_and_check():
        self.logger.error("Failed create new index [{0}]".format( self.new_index_name ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      if not self.add_write_disable_for_index_and_check():
        self.logger.error("Failed disable write to index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.index = self.new_index_name
      if not self.add_write_enable_for_index_and_check():
        self.logger.error("Failed enable write to next index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      if not self.ilm_retry_index_and_check():
        self.logger.error("Failed ilm retry to next index [{0}]".format( self.index ))

      self.index = index['index']
      self.not_next_step_for_index_and_check()

  def rollover_last_index_in_check_mode(self):
    for index in self.last_indices:
      self.logger.warning("[check_mode] Las index [{0}][{1}gb] is larger the allowed size - {2}gb".format( index['index'], int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))

  def rollover_not_last_index_in_check_mode(self):
    for index in self.not_last_indices:
      self.logger.warning("[check_mode] Not last index [{0}][{1}gb] is larger the allowed size - {2}gb".format( index['index'], int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))

  def rollover_last_shrink_index_in_check_mode(self):
    for index in self.last_shrink_indices:
      self.logger.warning("[check_mode] Last shrink index [{0}][{1}gb] is larger the allowed size - {2}gb".format( index['index'], int(index['pri.store.size']), self.MAX_CURRENT_INDEX_SIZE_GB ))



#!/usr/bin/python3

import re
import requests
from feii.log import Log
from feii.config import Config
from feii.init import Init
from feii.structure import Structure
from feii.function import Function

class Delete(Structure):
  def __init__(self,
    data_are_in_index: bool = False,
  ):
    super().__init__()
    self.data_are_in_index = data_are_in_index

  def debug_detail_index(self):
    self.index = 'test-000001'
    self.index_docs_count = 0

  def delete_not_last_indexes(self):
    for index in self.not_last_indices:
      self.index = index['index']
      if index['docs.count'] is None:
        continue

      if int(index['docs.count']) > 0:
        continue

      self.not_delete_index_and_check()

  def delete_not_last_indexes_check_mode(self):
    for index in self.not_last_indices:
      if index['docs.count'] is None:
        continue

      if int(index['docs.count']) > 0:
        continue

      self.logger.warning("[check_mode] Deletion an empty not last index [{0}], documents count - {1}".format( index['index'], index['docs.count'] ))

  def delete_last_indexes(self):
    for delete_index in self.delete_last_indices:
      self.index = delete_index['index']
      self.data_are_in_index = False

      if delete_index['docs.count'] is None:
        continue

      if int(delete_index['docs.count']) > 0:
        continue

      self.check_prev_indices()

      if not self.data_are_in_index:
        self.index = delete_index['index']
        self.not_delete_index_and_check()

  def delete_last_indexes_check_mode(self):
    for delete_index in self.delete_last_indices:
      self.index = delete_index['index']
      self.data_are_in_index = False

      if delete_index['docs.count'] is None:
        continue

      if int(delete_index['docs.count']) > 0:
        continue

      self.check_prev_indices()

      if not self.data_are_in_index:
        self.logger.warning("[check_mode] Deletion an empty last index [{0}], documents count - {1}".format( delete_index['index'], delete_index['docs.count'] ))

  def check_prev_indices(self):
    for x in range(3):
      self.prev_index_to_check = {}

      self.find_prev_index()
      self.without_shrink_prev_index = re.sub(r'(shrink-)', '', self.prev_index)

      if self.without_shrink_prev_index in Config.alias_list:
        self.creating_array_not_shrink_prev_index()

      if 'shrink-' + self.without_shrink_prev_index in Config.alias_list:
        self.creating_array_shrink_prev_index()

      self.check_empty_docs_in_prev_index()
      self.check_count_docs_in_prev_index()

      if not self.without_shrink_prev_index in Config.alias_list:
        self.index = self.without_shrink_prev_index
        continue

      if not 'shrink-' + self.without_shrink_prev_index in Config.alias_list:
        self.index = self.without_shrink_prev_index
        continue

  def check_empty_docs_in_prev_index(self):
    if 'docs.count' in self.prev_index_to_check and self.prev_index_to_check['docs.count'] is None:
      self.data_are_in_index = True

  def check_count_docs_in_prev_index(self):
    if 'docs.count' in self.prev_index_to_check and self.prev_index_to_check['docs.count'] != None and int(self.prev_index_to_check['docs.count']) > 0:
      self.data_are_in_index = True



#!/usr/bin/python3

import re
from feii.log import Log
from feii.config import Config
from feii.init import Init
from feii.structure import Structure

class Aliases(Structure):
  def add_alias_for_indices(self):
    for index in self.indices_no_alias:
      self.index = index['index']
      self.alias = index['index.alias']
      if self.add_alias_for_index_and_check():
        self.indices_not_write.append(index)

  def add_necessary_alias_for_indices(self):
    for index in self.indices_no_necessary_alias:
      self.index = index['index']
      self.alias = index['index.alias']
      if self.add_alias_for_index_and_check():
        self.indices_not_write.append(index)

  def add_alias_for_shrink_indices(self):
    for index in self.shrink_indices_no_alias:
      self.index = index['index']
      self.alias = re.sub(r'(shrink-)', '', index['index'])
      self.add_alias_for_index_and_check()

  def add_alias_for_indices_check_mode(self):
    for index in self.indices_no_alias:
      self.logger.warning("[check_mode] Adding an alias [{0}] for index [{1}]".format( index['index.alias'], index['index'] ))

  def add_necessary_alias_for_indices_check_mode(self):
    for index in self.indices_no_necessary_alias:
      self.logger.warning("[check_mode] Adding an alias [{0}] for index [{1}]".format( index['index.alias'], index['index'] ))

  def add_alias_for_shrink_indices_check_mode(self):
    for index in self.shrink_indices_no_alias:
      self.logger.warning("[check_mode] Adding an alias [{0}] for index [{1}]".format( re.sub(r'(shrink-)', '', index['index']), index['index'] ))

  def create_write_indices(self):
    for index in self.indices_not_write:
      self.index = index['index']
      self.alias = index['index.alias']

      if not self.add_write_enable_for_index_and_check():
        self.logger.error("Failed enable write to next index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

  def add_write_in_indices(self):
    for index in self.indices_not_write:
      self.index = index['index']
      self.alias = index['index.alias']

      if not self.add_write_disable_for_index_and_check():
        self.logger.error("Failed add alias [{0}] to index [{1}] and disable write".format( self.alias, self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.index = self.find_prev_index()
      if not self.add_write_disable_for_index_and_check():
        self.logger.error("Failed disable write to prev index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

      self.index = index['index']
      if not self.add_write_enable_for_index_and_check():
        self.logger.error("Failed enable write to index [{0}]".format( self.index ))
        self.logger.warning("Skip this index and continue with the following index")
        continue

  def add_write_in_indices_check_mode(self):
    for index in self.indices_not_write:
      self.logger.warning("[check_mode] Last index [{0}] no write capability".format( index['index'] ))



#!/usr/bin/python3

import click
from feii.cli import RolloverCLI

@click.command(
  cls=RolloverCLI,
  short_help='Rollover the big indexes.',
  epilog="Run 'feii rollover COMMAND --help' for more information on a command."
)
def cli():
  """Rollover the big indexes"""



#!/usr/bin/python3

import re
import time
import requests
from feii.log import Log
from feii.config import Config
from feii.init import Init
from feii.index import Index
from feii.shard import Shard
from feii.request import Request
from feii.ilm import Ilm
from feii.alias import Alias
from feii.cluster import Cluster
from feii.function import Function

class Structure(Index, Shard, Ilm, Alias, Function, Cluster, Request):
  def cluster_status(self):
    while True:
      self.get_status_cluster()

      if not self.check_count_relocating_shards_in_cluster():
        self.logger.warning("Number of relocating shards reaches {0}".format( self.cluster['relocating_shards'] ))
        self.retry += 1
        self.time_sleep()
        continue

      if not self.check_count_pending_tasks_in_cluster():
        self.logger.warning("Number of pending tasks reaches {0}".format( self.cluster['number_of_pending_tasks'] ))
        self.retry += 1
        self.time_sleep()
        continue

      if self.check_count_relocating_shards_in_cluster() and self.check_count_pending_tasks_in_cluster():
        self.retry = 0
        break

  def rollover_index_and_check(self):
    self.cluster_status()
    self.rollover_index()
    self.find_next_index()
    self.check_create_next_index()
    return self.check_rollover_index()

  def update_index_and_check(self):
    self.cluster_status()
    self.update_index()
    return self.check_update_index()

  def delete_index_and_check(self):
    self.delete_index()
    return self.check_delete_index()

  def reindexed_and_check(self):
    self.cluster_status()
    self.reindexed()
    return self.check_reindexed()

  def add_write_disable_for_index_and_check(self):
    self.add_alias_for_index_and_write_disable()
    self.cluster_status()
    self.request_add_alias_for_index()
    return self.check_disable_write_for_index()

  def add_write_enable_for_index_and_check(self):
    self.add_alias_for_index_and_write_enable()
    self.cluster_status()
    self.request_add_alias_for_index()
    return self.check_enable_write_for_index()

  def ilm_retry_index_and_check(self):
    self.cluster_status()
    self.ilm_retry_for_index()
    return self.check_ilm_retry_for_index()

  def next_step_for_index_and_check(self):
    self.current_ilm_info_for_index()
    self.create_current_ilm_info_for_index()
    self.next_step_for_not_shrink_index()
    self.next_step_for_shrink_index()
    self.cluster_status()
    self.request_step_for_index()
    return self.check_next_step_for_index()

  def re_step_for_index_and_check(self):
    self.current_ilm_info_for_index()
    self.create_current_ilm_info_for_index()
    self.next_step_index_in_warm()
    self.cluster_status()
    self.request_step_for_index()
    return self.check_re_step_for_index()

  def next_step_for_not_shrink_index(self):
    if not self.index_pattern.match(self.index).group(1):
      self.next_step_index_in_warm()

  def next_step_for_shrink_index(self):
    if self.index_pattern.match(self.index).group(1):
      self.next_step_index_in_cold()

  def not_next_step_for_index_and_check(self):
    if not self.next_step_for_index_and_check():
      self.logger.error("Failed next step to index [{0}]".format( self.index ))

  def not_delete_index_and_check(self):
    if not self.delete_index_and_check():
      self.logger.error("Failed delete index [{0}]".format( self.index ))

  def create_new_index_and_check(self):
    self.find_next_index()
    self.cluster_status()
    self.create_new_index()
    return self.check_create_new_index()

  def create_index_and_check(self):
    self.cluster_status()
    self.create_new_index()
    return self.check_create_new_index()

  def add_alias_for_index_and_check(self):
    self.add_alias_for_index()
    self.cluster_status()
    self.request_add_alias_for_index()
    return self.check_add_alias_for_index()



#!/usr/bin/python3

import os
from configparser import ConfigParser
from feii.config import Config

class PathToFile(Config):
  def __init__(self,
  ):
    super().__init__()

  def get_config(self):
    config_file = ConfigParser()
    config_file.read(self.path)
    self.t1 = config_file

    if config_file.has_option("CONFIG", "elastic_url"):
      Config.ELASTIC_URL = config_file.get("CONFIG", "elastic_url")

  def get_setting(self):
    if "CONFIG" in self.t1:
      for one, test in self.t1["CONFIG"].items():
        print("Name [{0}] - [{1}]".format( one, test ))



#!/usr/bin/python3

import click
from feii.cli import WriteCLI

@click.command(
  cls=WriteCLI,
  short_help='Correction of writing in indices.',
  epilog="Run 'feii write COMMAND --help' for more information on a command."
)
def cli():
  """Correction of writing in indices"""



#!/usr/bin/python3

import re
import requests
from feii.log import Log
from feii.config import Config
from feii.init import Init
from feii.request import Request
from feii.function import Function

class Alias(Config, Request):
  def __init__(self,
  ):
    super().__init__()

  def debug_detail_index(self):
    self.alias = 'test'
    self.index = 'test-000001'

  def add_alias_for_index(self):
    self.data = { "actions": [ { "add": { "index": self.index, "alias": self.alias } } ] }

  def add_alias_for_index_and_write_disable(self):
    self.data = { "actions": [ { "add": { "index": self.index, "alias": self.alias, "is_write_index": False } } ] }

  def add_alias_for_index_and_write_enable(self):
    self.data = { "actions": [ { "add": { "index": self.index, "alias": self.alias, "is_write_index": True } } ] }

  def request_add_alias_for_index(self):
    self.request = requests.post("{0}/_aliases?master_timeout={1}".format( self.ELASTIC_URL, self.MASTER_TIMEOUT ), json=self.data )

  def check_add_alias_for_index(self):
    if self.status_request():
      self.logger.info("Alias [{0}] was added to index [{1}]".format( self.alias, self.index ))
      return True

  def check_disable_write_for_index(self):
    if self.status_request():
      self.logger.info("Writing is disabled for the index [{0}]".format( self.index ))
      return True

  def check_enable_write_for_index(self):
    if self.status_request():
      self.logger.info("Writing is enabled for the index [{0}]".format( self.index ))
      return True



#!/usr/bin/python3

import feii
from setuptools import find_packages, setup

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = [
    "requests>=2",
    "click",
]

setup(
  name = "feii",
  version = feii.__version__,
  author = "Anton Turko",
  author_email = "anton_turko@mail.ru",
  url = "https://github.com/antohhh93/feii",
  description = "Fix error ilm index in Elasticsearch",
  long_description = readme,
  long_description_content_type = "text/markdown",
  packages = find_packages(),
  install_requires = requirements,
  license = "MIT",
  classifiers = [
      "Programming Language :: Python :: Implementation",
      "Programming Language :: Python :: 3",
      "Natural Language :: English",
      "License :: OSI Approved :: MIT License"
  ],
  entry_points = {
    "console_scripts": [
        "feii = feii.cli:cli",
    ],
  },
)



#!/usr/bin/python3

import click
from feii.main import class_structure, class_log, logging_level, updating_variables, generating_variables_for_indices, generating_variables_for_rollover, deleting_unnecessary_variables
from feii.rollover import Rollover

class_rollover = Rollover()
class_rollover.logger = class_log.logger

def start_rollover_all(check_mode):
  if not check_mode:
    class_rollover.rollover_last_index()
    class_rollover.rollover_not_last_index()
    class_rollover.rollover_last_shrink_index()

def start_check_mode_rollover_all(check_mode):
  if check_mode:
    class_rollover.rollover_last_index_in_check_mode()
    class_rollover.rollover_not_last_index_in_check_mode()
    class_rollover.rollover_last_shrink_index_in_check_mode()

@click.command(short_help='Runs all commands in order.')
@click.option(
  '-c', '--check-mode',
  is_flag=True,
  expose_value=True,
  help='Only displaying actions, without performing them.'
)
@click.option(
  '-l', '--log-level',
  default='info',
  show_default=True,
  expose_value=True,
  help='Set the logging level ("debug"|"info"|"warning"|"error"|"critical")'
)
@click.option(
  '-p', '--path_to_file',
  default='',
  expose_value=True,
  help='path'
)
def cli(check_mode, log_level, path_to_file):
  """Runs all commands in order"""

  logging_level(log_level)
  class_log.logger.info("Started rollover all big indexes")

  updating_variables(path_to_file)

  generating_variables_for_indices()
  generating_variables_for_rollover()
  deleting_unnecessary_variables()

  start_rollover_all(check_mode)
  start_check_mode_rollover_all(check_mode)

if __name__ == "__main__":
  cli()



#!/usr/bin/python3

import os
import click

cmd_folder = os.path.join(os.path.dirname(__file__), 'commands')
rollover_folder = os.path.join(os.path.dirname(__file__), 'commands/rollover')
alias_folder = os.path.join(os.path.dirname(__file__), 'commands/alias')
error_folder = os.path.join(os.path.dirname(__file__), 'commands/error')
delete_folder = os.path.join(os.path.dirname(__file__), 'commands/delete')
update_folder = os.path.join(os.path.dirname(__file__), 'commands/update')
write_folder = os.path.join(os.path.dirname(__file__), 'commands/write')

class AllCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(cmd_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class RolloverCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(rollover_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.rollover.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class AliasCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(alias_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.alias.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class ErrorCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(error_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.error.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class DeleteCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(delete_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.delete.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class UpdateCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(update_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.update.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

class WriteCLI(click.MultiCommand):
  def list_commands(self, ctx):
    rv = []
    for filename in os.listdir(write_folder):
      if filename.endswith('.py') and filename.startswith("cmd_"):
        rv.append(filename[4:-3])
    rv.sort()
    return rv

  def get_command(self, ctx, name):
    try:
      mod = __import__(f"feii.commands.write.cmd_{name}", None, None, ["cli"])
    except ImportError:
      return
    return mod.cli

@click.command(cls=AllCLI, epilog="Run 'feii COMMAND --help' for more information on a command.")
@click.version_option()
def cli():
  """
  \b
  Feii - utility for ilm indexes in Elasticsearch that allows you to:
    * fix errors in indexes
    * delete empty indexes
    * rollover big indexes
    * adding alias in indexes
    * applying parameters
    * fix write in indices
  """

if __name__ == '__main__':
  cli()
